from datetime import datetime, timedelta

from django.test import TestCase
from django_dynamic_fixture import G
import pytz

from ..models import LocalizedRecurrence, LocalizedRecurrenceQuerySet
from ..models import _replace_with_offset, _update_schedule


class LocalizedRecurrenceUpdateTest(TestCase):
    """
    Tests calling 'update' on a localized recurrence.
    """
    def test_update_creation(self):
        lr = LocalizedRecurrence()
        lr.update()
        self.assertIsNotNone(lr.id)

    def test_update_timezone(self):
        lr = G(LocalizedRecurrence)
        lr.update(timezone='US/Eastern')
        lr = LocalizedRecurrence.objects.get(id=lr.id)
        self.assertEqual(lr.timezone, pytz.timezone('US/Eastern'))

    def test_update_offset(self):
        lr = G(LocalizedRecurrence)
        lr.update(offset=timedelta(seconds=1))
        lr = LocalizedRecurrence.objects.get(id=lr.id)
        self.assertEqual(lr.offset, timedelta(seconds=1))


class LocalizedRecurrenceQuerySetTest(TestCase):
    """Simple test to ensure the custom query set is being used.
    """
    def test_isinstance(self):
        G(LocalizedRecurrence)
        recurrences = LocalizedRecurrence.objects.all()
        self.assertIsInstance(recurrences, LocalizedRecurrenceQuerySet)


class LocalizedRecurrenceQuerySetUpdateScheduleTest(TestCase):
    """Test that updates to recurrences are reflected in the DB.
    """
    def setUp(self):
        G(LocalizedRecurrence, interval='DAY', offset=timedelta(hours=12), timezone=pytz.timezone('US/Eastern'))
        G(LocalizedRecurrence, interval='MONTH', offset=timedelta(hours=15), timezone=pytz.timezone('US/Eastern'))

    def test_update_from_1970(self):
        """Start with next_scheduled of 1970, after update should be new.
        """
        time = datetime(year=2013, month=5, day=20, hour=12, minute=3)
        LocalizedRecurrence.objects.filter(interval='DAY').update_schedule(time=time)
        lr_day = LocalizedRecurrence.objects.filter(interval='DAY').first()
        self.assertGreater(lr_day.next_scheduled, time)


class LocalizedRecurrenceManagerUpdateScheduleTest(TestCase):
    def setUp(self):
        G(LocalizedRecurrence, interval='DAY', offset=timedelta(hours=12), timezone=pytz.timezone('US/Eastern'))
        G(LocalizedRecurrence, interval='MONTH', offset=timedelta(hours=15), timezone=pytz.timezone('US/Eastern'))

    def test_update_all(self):
        """Calls to the model manager to update should be passed through.
        """
        time = datetime(year=2013, month=5, day=20, hour=15, minute=3)
        LocalizedRecurrence.objects.update_schedule(time=time)
        self.assertTrue(all(r.next_scheduled > time for r in LocalizedRecurrence.objects.all()))


class LocalizedRecurrenceTest(TestCase):
    """Test the creation and querying of LocalizedRecurrence records.
    """
    def setUp(self):
        G(LocalizedRecurrence, interval='DAY', offset=timedelta(hours=12), timezone=pytz.timezone('US/Eastern'))

    def test_timedelta_returned(self):
        """Test that the Duration field is correctly returning timedeltas.
        """
        lr = LocalizedRecurrence.objects.first()
        self.assertTrue(isinstance(lr.offset, timedelta))

    def test_string_representation(self):
        lr = LocalizedRecurrence.objects.first()
        self.assertEqual(
            str(lr),
            'ID: {0}, Interval: {1}, Next Scheduled: {2}'.format(lr.id, lr.interval, lr.next_scheduled),
        )


class LocalizedRecurrenceUpdateScheduleTest(TestCase):
    def setUp(self):
        self.lr_day = G(LocalizedRecurrence,
                        interval='DAY', offset=timedelta(hours=12), timezone=pytz.timezone('US/Eastern'))

    def test_update_passes_through(self):
        time = datetime(year=2013, month=5, day=20, hour=15, minute=3)
        self.lr_day.update_schedule(time)
        self.assertGreater(self.lr_day.next_scheduled, time)


class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
    def setUp(self):
        self.lr_day = G(
            LocalizedRecurrence,
            interval='DAY', offset=timedelta(hours=12),
            timezone=pytz.timezone('US/Eastern'))
        self.lr_week = G(
            LocalizedRecurrence,
            interval='WEEK', offset=timedelta(days=3, hours=17, minutes=30),
            timezone=pytz.timezone('US/Central'))
        self.lr_month = G(
            LocalizedRecurrence,
            interval='MONTH', offset=timedelta(days=21, hours=19, minutes=15, seconds=10),
            timezone=pytz.timezone('US/Central'))
        self.lr_quarter = G(
            LocalizedRecurrence,
            interval='QUARTER', offset=timedelta(days=68, hours=16, minutes=30),
            timezone=pytz.timezone('Asia/Hong_Kong'))
        self.lr_year = G(
            LocalizedRecurrence,
            interval='YEAR', offset=timedelta(days=31, hours=16, minutes=30),
            timezone=pytz.timezone('Asia/Hong_Kong'))

    def test_basic_works(self):
        """Test a simple case of utc_of_next_schedule.

        - The given recurrence is scheduled daily for Eastern Time at noon.
        - The given current date in UTC is 2013/1/15::17:05:22
        - We then expect the next schedule in UTC to be 2013/1/16::17:0:0
        """
        current_time = datetime(2013, 1, 15, 17, 5, 22)
        expected_next_schedule = datetime(2013, 1, 16, 17)
        schedule_out = self.lr_day.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_dst_cross_monthly(self):
        """The case when a monthly recurrence goes past daylight savings time"""
        self.lr_month.offset = timedelta(hours=0)
        self.lr_month.previous_scheduled = datetime(2015, 2, 1, 6)
        scheduled_out = self.lr_month.utc_of_next_schedule(datetime(2015, 3, 31))
        self.assertEqual(scheduled_out, datetime(2015, 4, 1, 5))

    def test_before_midnight(self):
        """The case when the scheduled and current time cross midnight.

        - The given recurrence is scheduled daily for Eastern Time at 11:59PM.
        - The given current date in UTC is 2013/1/15::05:05:22 -> 12:05AM EST
        - We then expect the next schedule in UTC to be 2013/1/16::05:0:0
        """
        self.lr_day.offset = timedelta(hours=23, minutes=59)
        self.lr_day.save()
        current_time = datetime(2013, 1, 15, 5, 5, 22)
        expected_next_schedule = datetime(2013, 1, 16, 4, 59)
        schedule_out = self.lr_day.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_after_midnight(self):
        """The case when the scheduled and current time are after midnight.

        - The given recurrence is scheduled daily for Eastern Time at 12:01 AM.
        - The given current date in UTC is 2013/1/15::05:05:22 -> 12:05AM EST
        - We then expect the next schedule in UTC to be 2013/1/16::05:01:0
        """
        self.lr_day.offset = timedelta(minutes=1)
        self.lr_day.save()
        current_time = datetime(2013, 1, 15, 5, 5, 22)
        expected_next_schedule = datetime(2013, 1, 16, 5, 1)
        schedule_out = self.lr_day.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_week_update_full_week(self):
        """Weekly Recurrences should be able to add a full week.

        - Thursday August 8th at 10:34 PM UTC is 5:34 PM CDT.
        - Scheduled for weekly, Thursday at 5:30
        - Expect next schedule to be Thursday August 15th at 10:30 PM UTC
        """
        current_time = datetime(2013, 8, 8, 22, 34)
        expected_next_schedule = datetime(2013, 8, 15, 22, 30)
        schedule_out = self.lr_week.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_weekly_update_current_week(self):
        """Weekly Recurrences should be able to work in the current week.

        - Tuesday August 6th at 10:34 PM UTC is 5:34 PM CDT.
        - Scheduled for weekly, Thursday at 5:30
        - Expect next schedule to be Thursday August 8th at 10:30 PM UTC
        """
        current_time = datetime(2013, 8, 6, 22, 34)
        expected_next_schedule = datetime(2013, 8, 8, 22, 30)
        schedule_out = self.lr_week.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_month_update_full_month(self):
        """Monthly Recurrences should work as expected moving forward a full month.

        - Friday August 23rd at 12:34 AM UTC is Thursday August 22nd at 7:34 PM CDT.
        - Scheduled for Monthly, on the 21st day at 7:15.10 PM
        - Expect next schedule to be September 23 at 12:15.10 AM UTC
        """
        current_time = datetime(2013, 8, 23, 0, 34, 55)
        expected_next_schedule = datetime(2013, 9, 23, 0, 15, 10)
        schedule_out = self.lr_month.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_month_update_current_month(self):
        """Monthly Recurrences should work as expected moving forward in the
        current month.

        - Friday August 16th at 12:34 AM UTC is Thursday August 15th at 7:34 PM CDT.
        - Scheduled for Monthly, on the 21st day at 7:15.10 PM
        - Expect next schedule to be August 23 at 12:15.10 AM UTC

        """
        current_time = datetime(2013, 8, 16, 0, 34, 55)
        expected_next_schedule = datetime(2013, 8, 23, 0, 15, 10)
        schedule_out = self.lr_month.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_month_year_end_update(self):
        """Monthly Recurrences should transition over years correctly

        - Monday December 23 at 2:34 AM UTC is Sunday August 22nd at 9:34 PM CDT.
        - Scheduled for Monthly, on the 21st day at 7:15.10 PM CDT
        - Expect next schedule to be January 23 at 1:15.10 AM UCT
        """
        current_time = datetime(2013, 12, 23, 2, 34, 55)
        expected_next_schedule = datetime(2014, 1, 23, 1, 15, 10)
        schedule_out = self.lr_month.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_quarterly_full_quarter(self):
        """Quarterly Recurrences should be able to update a full quarter.

        - June 23rd at 12:34 AM UTC is June 22nd at 10:34 PM HKT.
        - Scheduled for Quarterly, on the 68th day at 4:30 PM HKT
        - Expect next schedule to be September 8th at 8:30 AM UTC
        """
        current_time = datetime(2013, 6, 23, 0, 34, 55)
        expected_next_schedule = datetime(2013, 9, 8, 8, 30)
        schedule_out = self.lr_quarter.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_quarterly_current_quarter(self):
        """Quarterly Recurrences should be able to update in the current quarter.

        - April 23rd at 12:34 AM UTC is April 22nd at 10:34 PM HKT.
        - Scheduled for Quarterly, on the 68th day at 4:30 PM HKT
        - Expect next schedule to be June 8th at 8:30 AM UTC
        """
        current_time = datetime(2013, 4, 23, 0, 34, 55)
        expected_next_schedule = datetime(2013, 6, 8, 8, 30)
        schedule_out = self.lr_quarter.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_quarterly_end_year(self):
        """Quarterly Recurrences should be able to update through year end.

        - December 23rd at 12:34 AM UTC is April 22nd at 10:34 PM HKT.
        - Scheduled for Quarterly, on the 68th day at 4:30 PM HKT
        - Expect next schedule to be March 8th at 8:30 AM UTC
        """
        current_time = datetime(2013, 12, 23, 0, 34, 55)
        expected_next_schedule = datetime(2014, 3, 8, 8, 30)
        schedule_out = self.lr_quarter.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_yearly(self):
        """Yearly Recurrences should work as expected.

        - June 23rd at 12:34 AM UTC is June 22nd at 10:34 PM HKT.
        - Scheduled for Yearly, on the 31st day at 4:30 PM HKT
        - Expect next schedule to be February 1st at 8:30 AM UTC
        """
        current_time = datetime(2013, 6, 23, 0, 34, 55)
        expected_next_schedule = datetime(2014, 2, 1, 8, 30)
        schedule_out = self.lr_year.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_into_dst_boundary(self):
        """Recurrences happen at the correct local time after going into DST.

        Going into daylight savings time should mean the UTC time is
        an hour less on the next recurrence.

        - 2013 US DST began: Sunday March 10th.
        - Weekly recurence, every Thursday at Noon EST.
        """
        self.lr_week.offset = timedelta(days=3, hours=12)
        self.lr_week.save()
        current_time = datetime(2013, 3, 7, 18)
        expected_next_schedule = datetime(2013, 3, 14, 17)
        schedule_out = self.lr_week.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_out_of_dst_boundary(self):
        """Recurrences at the correct local time after going out of UTC.

        Going into daylight savings time should mean the UTC time is
        an hour greater on the next recurrence.

        - 2013 US DST ended: Sunday November 3rd.
        - Weekly recurence, every Thursday at Noon EST.
        """
        self.lr_week.offset = timedelta(days=3, hours=12)
        self.lr_week.save()
        current_time = datetime(2013, 10, 31, 17)
        expected_next_schedule = datetime(2013, 11, 7, 18)
        schedule_out = self.lr_week.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)

    def test_utc_plus(self):
        """Test a timezone that is UTC + 2.

        - Europe/Berlin DST is UTC + 2
        """
        self.lr_day.timezone = pytz.timezone('Europe/Berlin')
        self.lr_day.save()
        current_time = datetime(2013, 5, 5, 10, 10)
        expected_next_schedule = datetime(2013, 5, 6, 10)
        schedule_out = self.lr_day.utc_of_next_schedule(current_time)
        self.assertEqual(schedule_out, expected_next_schedule)


class UpdateScheduleTest(TestCase):
    def setUp(self):
        self.lr_week = G(
            LocalizedRecurrence,
            interval='WEEK', offset=timedelta(hours=12),
            timezone=pytz.timezone('US/Eastern'))
        self.lr_day = G(
            LocalizedRecurrence,
            interval='DAY',
            offset=timedelta(hours=12),
            timezone=pytz.timezone('US/Eastern'))

    def test_updates_localized_recurrences(self):
        time = datetime(year=2013, month=5, day=20, hour=12, minute=3)
        _update_schedule([self.lr_week], time)
        self.assertGreater(self.lr_week.next_scheduled, time)
        self.assertEqual(self.lr_week.previous_scheduled, time)


class ReplaceWithOffsetTest(TestCase):
    def test_day(self):
        """_replace_with_offset works as expected with a 'DAY' interval.
        """
        dt_in = datetime(2013, 1, 20, 12, 45, 48)
        td_in = timedelta(hours=3, minutes=3, seconds=3)
        interval_in = 'DAY'
        dt_expected = datetime(2013, 1, 20, 3, 3, 3)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_out, dt_expected)

    def test_week(self):
        """_replace_with_offset works as expected with a 'WEEK' interval.
        """
        dt_in = datetime(2013, 1, 20, 12, 45, 48)
        td_in = timedelta(days=4, hours=3, minutes=3, seconds=3)
        interval_in = 'WEEK'
        dt_expected = datetime(2013, 1, 18, 3, 3, 3)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_out, dt_expected)

    def test_week_on_month_boundary(self):
        """_replace_with_offset using interval 'WEEK' should roll over months
        correctly.
        """
        dt_in = datetime(2013, 7, 30, 12, 45, 48)
        td_in = timedelta(days=4, hours=3, minutes=3, seconds=3)
        interval_in = 'WEEK'
        dt_expected = datetime(2013, 8, 2, 3, 3, 3)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_out, dt_expected)

    def test_month(self):
        """_replace_with_offset works as expected with a 'MONTH' interval.
        """
        dt_in = datetime(2013, 1, 20, 12, 45, 48)
        td_in = timedelta(days=15, hours=3, minutes=3, seconds=3)
        interval_in = 'MONTH'
        dt_expected = datetime(2013, 1, 16, 3, 3, 3)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_out, dt_expected)

    def test_last_day_of_month(self):
        dt_in = datetime(2013, 2, 20, 12, 45, 48)
        td_in = timedelta(days=31, hours=3, minutes=3, seconds=3)
        interval_in = 'MONTH'
        dt_expected = datetime(2013, 2, 28, 3, 3, 3)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_out, dt_expected)

    def test_quarter(self):
        dt_in = datetime(2013, 4, 20, 12, 45, 48)
        td_in = timedelta(days=65, hours=3, minutes=3, seconds=3)
        interval_in = 'QUARTER'
        dt_expected = datetime(2013, 6, 5, 3, 3, 3)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_out, dt_expected)

    def test_quarterly_past(self):
        dt_in = datetime(2013, 6, 23, 0, 34, 55)
        td_in = timedelta(days=68, hours=16, minutes=30)
        interval_in = 'QUARTER'
        dt_expected = datetime(2013, 6, 8, 16, 30)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_expected, dt_out)

    def test_quarterly_overshoot(self):
        dt_in = datetime(2013, 1, 1, 0)
        td_in = timedelta(days=90, hours=12)
        interval_in = 'QUARTER'
        dt_expected = datetime(2013, 3, 31, 12)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_expected, dt_out)

    def test_quarterly_undershoot(self):
        dt_in = datetime(2013, 7, 1, 0)
        td_in = timedelta(days=90, hours=12)
        interval_in = 'QUARTER'
        dt_expected = datetime(2013, 9, 29, 12)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_expected, dt_out)

    def test_year(self):
        dt_in = datetime(2013, 6, 23, 0, 34, 55)
        td_in = timedelta(days=5, hours=16, minutes=30)
        interval_in = 'YEAR'
        dt_expected = datetime(2013, 1, 6, 16, 30)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_expected, dt_out)

    def test_year_end_leap_year(self):
        dt_in = datetime(2016, 6, 23, 0, 34, 55)
        td_in = timedelta(days=365, hours=16, minutes=30)
        interval_in = 'YEAR'
        dt_expected = datetime(2016, 12, 31, 16, 30)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_expected, dt_out)

    def test_year_end_non_leap_year(self):
        dt_in = datetime(2015, 6, 23, 0, 34, 55)
        td_in = timedelta(days=365, hours=16, minutes=30)
        interval_in = 'YEAR'
        dt_expected = datetime(2015, 12, 31, 16, 30)
        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
        self.assertEqual(dt_expected, dt_out)

    def test_bad_interval(self):
        """A missformed interval should raise a value error
        """
        dt_in = datetime(2013, 1, 20, 12, 45, 48)
        td_in = timedelta(days=15, hours=3, minutes=3, seconds=3)
        interval_in = 'blah'
        with self.assertRaises(ValueError):
            _replace_with_offset(dt_in, td_in, interval_in)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-08 21:17
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('localized_recurrence', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='localizedrecurrence',
            name='offset2',
            field=models.DurationField(default=datetime.timedelta(0)),
        ),
    ]



import unittest
from datetime import timedelta

import django
import six
if six.PY3:  # pragma: no cover
    from importlib import reload

from mock import MagicMock, patch

from .. import fields


class DurationFieldToPythonTest(unittest.TestCase):
    def setUp(self):
        """Create a mock of the DurationField class and pin to_python to it.
        """
        self.df = fields.DurationField()
        self.df_with_default = fields.DurationField(default=60)

    def test_deconstruct_with_default(self):
        """
        Test .deconstruct() with a default
        """
        if django.VERSION[1] >= 7:  # pragma: no cover
            name, path, args, kwargs = self.df_with_default.deconstruct()

            self.assertEqual(kwargs['default'], 60)
        else:  # pragma: no cover
            with patch('localized_recurrence.fields.IntegerField.deconstruct', create=True) as mock_deconstruct:
                mock_deconstruct.return_value = (
                    'duration', 'django.db.models.IntegerField', [], {'default': timedelta(seconds=60)}
                )
                name, path, args, kwargs = self.df_with_default.deconstruct()
                self.assertEqual(kwargs['default'], 60)

    def test_deconstruct_without_default(self):
        """
        Test .deconstruct() without a default
        """
        if django.VERSION[1] >= 7:  # pragma: no cover
            name, path, args, kwargs = self.df.deconstruct()
            self.assertFalse('default' in kwargs.keys())
        else:  # pragma: no cover
            with patch('localized_recurrence.fields.IntegerField.deconstruct', create=True) as mock_deconstruct:
                mock_deconstruct.return_value = (
                    'duration', 'django.db.models.IntegerField', [], {}
                )
                name, path, args, kwargs = self.df_with_default.deconstruct()
                self.assertNotIn('default', kwargs.keys())

    def test_timedelta(self):
        """A timedelta should just get returned.
        """
        td_in = timedelta(days=1, hours=3)
        td_out = self.df.to_python(td_in)
        self.assertEqual(td_out, td_in)

    def test_string(self):
        """A string should be properly converted.
        """
        td_expected = timedelta(days=3, hours=5, minutes=10)
        str_in = str(td_expected)
        td_out = self.df.to_python(str_in)
        self.assertEqual(td_out, td_expected)

    def test_bad_string(self):
        """Malformed strings should raise an error.
        """
        str_in = "3:5"
        with self.assertRaises(ValueError):
            self.df.to_python(str_in)

    def test_int(self):
        """Int inputs should give timedelta outputs
        """
        int_in = 176400
        td_expected = timedelta(days=2, hours=1)
        td_out = self.df.to_python(int_in)
        self.assertEqual(td_out, td_expected)

    def test_none(self):
        """Null input -> None output
        """
        td_in = None
        td_out = self.df.to_python(td_in)
        self.assertEqual(td_in, td_out)

    def test_invalid(self):
        """Unsupported objects raise ValueError
        """
        td_in = self
        with self.assertRaises(ValueError):
            self.df.to_python(td_in)


class DurationFieldGetPrepValueTest(unittest.TestCase):
    def setUp(self):
        """Create a mock of the DurationField class and pin get_prep_value.
        """
        self.df = fields.DurationField()

    def test_returns_int(self):
        """Type should be `int`.
        """
        td_in = timedelta(hours=14)
        int_out = self.df.get_prep_value(td_in)
        self.assertTrue(isinstance(int_out, int))

    def test_returns_int2(self):
        """Type should be `int`.
        """
        td_in = timedelta(hours=14, seconds=3.5)
        int_out = self.df.get_prep_value(td_in)
        self.assertTrue(isinstance(int_out, int))

    def test_round_trip(self):
        """A trip through get_prep_value and to_python.
        """
        td_in = timedelta(hours=14)
        int_out = self.df.get_prep_value(td_in)
        td_out = self.df.to_python(int_out)
        self.assertEqual(td_out, td_in)


class DurationFieldValueToStringTest(unittest.TestCase):
    def setUp(self):
        """Create a mock of the DurationField class and pin value_to_string to it.

        We also mock out the _get_val_from_obj, to just return what is
        passed in. Allows us to pass in a timedelta for testing intead
        of the expected object.
        """
        self.mock_DurationField = MagicMock()
        self.mock_DurationField.value_to_string = fields.DurationField.__dict__['value_to_string']
        self.mock_DurationField.to_python = fields.DurationField.__dict__['to_python']
        self.mock_DurationField._get_val_from_obj.side_effect = lambda x: x

    def test_simple_string(self):
        """
        """
        in_td = timedelta(days=1, hours=1, minutes=1, seconds=1)
        expected_str = "1 day, 1:01:01"
        out_str = self.mock_DurationField.value_to_string(self.mock_DurationField, in_td)
        self.assertEqual(out_str, expected_str)

    def test_loop_to_python(self):
        """A roundtrip through value_to_string and to_python.

        We expect the value to be unchanged after the trip.
        """
        in_td = timedelta(days=1, hours=1, minutes=1, seconds=1)
        out_str = self.mock_DurationField.value_to_string(self.mock_DurationField, in_td)
        out_td = self.mock_DurationField.to_python(self.mock_DurationField, out_str)
        self.assertEqual(out_td, in_td)


class ParseTimedeltaStringTest(unittest.TestCase):
    """Test some round trips between timedelta, string, timedelta.
    """
    def test_works(self):
        """Test a bunch of cases for the regular expressions.

        We test so many things in this test because regular
        expressions represent a lot of effective branching.
        """
        td1_in = timedelta(days=1)
        td2_in = timedelta(seconds=1)
        td3_in = timedelta(hours=1, minutes=1, seconds=1)
        td4_in = timedelta(days=1, seconds=10000.001)
        td5_in = timedelta(hours=12)
        td6_in = timedelta(hours=-12)
        str1_out = str(td1_in)
        str2_out = str(td2_in)
        str3_out = str(td3_in)
        str4_out = str(td4_in)
        str5_out = str(td5_in)
        str6_out = str(td6_in)
        td1_out = fields.parse_timedelta_string(str1_out)
        td2_out = fields.parse_timedelta_string(str2_out)
        td3_out = fields.parse_timedelta_string(str3_out)
        td4_out = fields.parse_timedelta_string(str4_out)
        td5_out = fields.parse_timedelta_string(str5_out)
        td6_out = fields.parse_timedelta_string(str6_out)
        self.assertEqual(td1_in, td1_out)
        self.assertEqual(td2_in, td2_out)
        self.assertEqual(td3_in, td3_out)
        self.assertEqual(td4_in, td4_out)
        self.assertEqual(td5_in, td5_out)
        self.assertEqual(td6_in, td6_out)


class SetupSouthTest(unittest.TestCase):
    def test_no_south(self):
        """This test is meant to hit the branch handleing the ImportError for
        south.

        Some users may not have south, so if it is not available, we
        just pass.
        """
        with patch.dict('sys.modules', {'south.modelsinspector': {}}):
            reload(fields)



from django.contrib.admin import ModelAdmin, site

from localized_recurrence.models import LocalizedRecurrence


class LocalizedRecurrenceAdmin(ModelAdmin):
    list_display = [
        'id',
        'interval',
        'timezone',
        'offset',
        'previous_scheduled',
        'next_scheduled',
    ]


site.register(LocalizedRecurrence, LocalizedRecurrenceAdmin)



from django.contrib.admin.sites import AdminSite
from django.test import TestCase

from localized_recurrence.admin import LocalizedRecurrenceAdmin
from localized_recurrence.models import LocalizedRecurrence


class LocalizedRecurrenceAdminTest(TestCase):
    """Verify that the admin can load.
    """
    def setUp(self):
        super(LocalizedRecurrenceAdminTest, self).setUp()
        self.site = AdminSite()

    def test_model_admin_load(self):
        lr_admin = LocalizedRecurrenceAdmin(LocalizedRecurrence, self.site)
        self.assertEqual(
            lr_admin.list_display,
            [
                'id',
                'interval',
                'timezone',
                'offset',
                'previous_scheduled',
                'next_scheduled'
            ]
        )



from datetime import timedelta
import re

from django.db.models.fields import IntegerField
import six


class CastOnAssignDescriptor(object):
    """
    A property descriptor which ensures that `field.to_python()` is called on _every_ assignment to the field.
    This used to be provided by the `django.db.models.subclassing.Creator` class, which in turn
    was used by the deprecated-in-Django-1.10 `SubfieldBase` class, hence the reimplementation here.
    https://stackoverflow.com/questions/39392343/
    how-do-i-make-a-custom-model-field-call-to-python-when-the-field-is-accessed-imm
    """

    def __init__(self, field):
        self.field = field

    def __get__(self, obj, type=None):  # pragma: no cover
        if obj is None:
            return self
        return obj.__dict__[self.field.name]

    def __set__(self, obj, value):  # pragma: no cover
        obj.__dict__[self.field.name] = self.field.to_python(value)


class DurationField(IntegerField):
    """A field to store durations of time with accuracy to the second.

    A Duration Field will automatically convert between python
    timedelta objects and database integers.

    Duration fields can be used to define fields in a django model

    .. code-block:: python

        class Presentations(models.Model):
            length = DurationField()
            speaker = models.ForeignKey(User)
            location = models.ForeignKey(Location)

    Given such a model, storing a duration in the database is as
    simple as passing in a ``timedelta`` object

    .. code-block:: python

        >>> Presentations.objects.create(
        ...     length=datetime.timedelta(minutes=45),
        ...     speaker=User.objects.get(email='MrT@example.com'),
        ...     location=wrestle_mania_ring,
        ... )

    The timedeltas are stored as seconds in the backend, so sub-second
    accuracy is lost with this field.

    """
    description = "A duration of time."

    def __init__(self, *args, **kwargs):
        """Call out to the super. Makes docs cleaner."""
        return super(DurationField, self).__init__(*args, **kwargs)

    def contribute_to_class(self, cls, name):
        super(DurationField, self).contribute_to_class(cls, name)
        setattr(cls, name, CastOnAssignDescriptor(self))

    def to_python(self, value):
        """Convert a stored duration into a python datetime.timedelta object.

        Need to handle three cases:
            - An instance of the correct type (timedelta).
            - A string (e.g., from a deserializer).
            - What the database returns (integer).
        """
        if isinstance(value, timedelta):
            v = value
        elif isinstance(value, (six.binary_type, six.text_type)):
            # The string should be in the form "[D day[s],][H]H:MM:SS[.UUUUUU]"
            try:
                v = parse_timedelta_string(value)
            except ValueError:
                raise ValueError("Duration string must be in the form '[D day[s],][H]H:MM:SS[.UUUUUU]'")
        elif isinstance(value, int):
            v = timedelta(seconds=value)
        elif value is None:
            v = None
        else:
            raise ValueError("Not a valid Duration object")
        return v

    def deconstruct(self):
        name, path, args, kwargs = super(DurationField, self).deconstruct()
        if 'default' in kwargs.keys():
            kwargs['default'] = self.get_prep_value(kwargs['default'])
        return name, 'localized_recurrence.fields.DurationField', args, kwargs

    def get_prep_value(self, value):
        """Convert a timedelta to integer for storage.
        """
        value = self.to_python(value)
        return int(value.total_seconds())

    def value_to_string(self, obj):
        """Used by serializers to get a string representation.
        """
        time_delta_value = self._get_val_from_obj(obj)
        return str(time_delta_value)


def parse_timedelta_string(string):
    """Parses strings from datetime.timedelta.__str__.

    Usefull because django's force_text uses .__str__ instead of
    .__repr__

    datetime.timedelta.__str__ returns a string in the form [D day[s],
    ][H]H:MM:SS[.UUUUUU], where D is negative for negative t.
    """
    days_re = "(?P<days>-?[0-9]*) days?, (?P<hours>[0-9]+):(?P<minutes>[0-9]+):(?P<seconds>[0-9]+\.?[0-9]*)"
    no_days_re = "(?P<hours>[0-9]+):(?P<minutes>[0-9]+):(?P<seconds>[0-9]+\.?[0-9]*)"
    match_days = re.match(days_re, string)
    match_no_days = re.match(no_days_re, string)
    if match_days:
        return timedelta(**{k: float(v) for k, v in match_days.groupdict().items()})
    elif match_no_days:
        return timedelta(**{k: float(v) for k, v in match_no_days.groupdict().items()})
    else:
        raise ValueError("'%s' is not in the form [D day[s],][H]H:MM:SS[.UUUUUU]" % string)



import re
from setuptools import setup, find_packages

import multiprocessing
assert multiprocessing


def get_version():
    """
    Extracts the version number from the version.py file.
    """
    VERSION_FILE = 'localized_recurrence/version.py'
    mo = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', open(VERSION_FILE, 'rt').read(), re.M)
    if mo:
        return mo.group(1)
    else:
        raise RuntimeError('Unable to find version string in {0}.'.format(VERSION_FILE))


setup(
    name='django-localized-recurrence',
    version=get_version(),
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    license='MIT License',
    description='Store events that recur in users\' local times.',
    long_description=open('README.rst').read(),
    author='Erik Swanson',
    author_email='theerikswanson@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'Django>=1.9',
        'ambition-django-timezone-field>=2.0.1',
        'fleming>=0.4.6',
        'python-dateutil',
        'pytz',
    ],
    tests_require=[
        'django-nose>=1.4',
        'django-dynamic-fixture',
        'mock',
    ],
    test_suite='run_tests.run_tests'
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import timezone_field.fields
import localized_recurrence.fields
import datetime


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LocalizedRecurrence',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('interval', models.CharField(default='DAY', choices=[('DAY', 'Day'), ('WEEK', 'Week'), ('MONTH', 'Month'), ('QUARTER', 'Quarter'), ('YEAR', 'Year')], max_length=18)),
                ('offset', localized_recurrence.fields.DurationField(default=0)),
                ('timezone', timezone_field.fields.TimeZoneField(default='UTC')),
                ('previous_scheduled', models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0))),
                ('next_scheduled', models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0))),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from datetime import timedelta
from django.db import migrations


def migrate_integers_to_intervals(apps, schema_editor):
    # Migrate custom DurationField values to Django's Native DurationField values
    LocalizedRecurrence = apps.get_model('localized_recurrence', 'LocalizedRecurrence')
    for lr in LocalizedRecurrence.objects.all():
        lr.offset2 = lr.offset
        lr.save()


class Migration(migrations.Migration):
    dependencies = [
        ('localized_recurrence', '0002_localizedrecurrence_offset2'),
    ]

    operations = [
        migrations.RunPython(migrate_integers_to_intervals),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-08 21:51
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('localized_recurrence', '0003_offset_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='localizedrecurrence',
            name='offset',
        ),
        migrations.RenameField(
            model_name='localizedrecurrence',
            old_name='offset2',
            new_name='offset',
        ),
    ]
